/** 
*  \file Monitor.h 
*  \brief contains toolset for monitoring subsystem.
*
*  \author M.Smuga-Otto <maciek@ssec.wisc.edu>, R.K.Garcia <rayg@ssec.wisc.edu>
*
*  \version $Id: Monitor.h,v 1.4 2004/10/07 15:48:25 maciek Exp $
*
*  \verbatim
*
*  Monitoring ports are implemented in C++ as ostream-compatible object references.
*  A means of providing useful shorthand to the enclosing system is desirable.
*  For the 'testing' implementation of monitoring, the ostreams and the shorthand
*  passed to them is shifted to simple text and sent on through a trivial ostream
*  such as cerr. 
*
*  Specializations of the monitoring module are done by re-implementing Monitor.cc
*  and linking in a Monitor.o specific to the enclosing framework. This way the
*  customization of the 'spanning' concern of monitoring the operators is tractable.
*
*  Example.
*  Operator is forced an unexpected re-planning of its FFT.
*  This becomes a performance warning which is queued to the monitoring system.
* 
*     mon_stream << mon::warning( "FFT plan rebuild required", __FILE__, __LINE__ ) << mon::end;
*
*  Copyright UW/SSEC, ALL RIGHTS RESERVED, 2004
*  Space Science and Engineering Center
*  University of Wisconsin - Madison, USA
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
*  \endverbatim
*/


namespace mon
{
    // FIXME: There should be a more elegant way to compose shorthand elements using templates with typelists.
    
    /// policy enforcing that this element be streamable
    template< typename ShorthandElement >
    class sendable 
    {
    public:
        static std::ostream &operator<<( std::ostream &, const ShorthandElement & );
    };
    
    
    struct _basicMessage
    {
        const char * const desc_;
        const char * const file_;
        const int line_;
    };
    
    class warning: sendable<_basicMessage> 
    {
    public:
         warning( const char const * &desc, const char const *file, int line ): 
        desc_(desc), file_(file), line_(line) { }
        // implement operator<< in Monitor.cc
    };




};
