/**
 * \file Operator.h
 * \brief contains abstraoct operator definition
 * \author Ray Garcia <rayg@ssec.wisc.edu>, Maciek Smuga-Otto <maciek@ssec.wisc.edu>
 *
 *  \version $Id: Operator.h,v 1.5 2004/10/07 15:48:25 maciek Exp $
 *
 *  \par Copyright:
 *  \verbatim
 *
 *  Copyright UW/SSEC, ALL RIGHTS RESERVED, 2004
 *  Space Science and Engineering Center
 *  University of Wisconsin - Madison, USA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 *  \endverbatim
 */


#ifndef H_OPERATOR
#define H_OPERATOR

namespace Operator 
{

/** Standardized return values for abstract operators 
*/
    
// replace this with something that can do cumulative checking
    // ie that a single warning or error sticks...
enum Result 
{
    UNIMPLEMENTED=-2, /// construction stopgap
    ERROR=-1, /// Could not run to completion
    OK=0,     /// Ran to completion
    WARNING=1 /// Ran to completion, results may not be useful.
};

class AbstractOperator
{

public:

    /** Invocation operator. 
    * Fetch input and output buffers from Ports, 
    *   once and only once per invocation.
    * Do the computation.
    * Return a valid result code.
    */
    virtual Result operator()( ) = 0;

}; // class AbstractOperator


}; // namespace

#endif