/**
 * \file IfgShift.h
 * \brief Shifts raw interferogram to form expected by the FFT operator.
 * \author Maciek Smuga-Otto <maciek@ssec.wisc.edu>
 *
 * based directly on the Scanning-HIS rsh2fbf application, written by Ray Garcia <rayg@ssec.wisc.edu>
 *
 *  \version $Id: IfgShift.h,v 1.12 2004/10/07 15:53:33 maciek Exp $
 *
 *  \par Copyright:
 *  \verbatim
 *
 *  Copyright UW/SSEC, ALL RIGHTS RESERVED, 2004
 *  Space Science and Engineering Center
 *  University of Wisconsin - Madison, USA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 *  \endverbatim
 */

#ifndef H_IFGSHIFT
#define H_IFGSHIFT

#ifndef NO_DEPENDENCY_INCLUDES
// GIPS includes
#include "Physics.h"
#include "Instrument.h"
#include "ProtocolRole.h"
#include "Operator.h"

#endif

namespace IfgShift 
{
    
/// configuration structure for the Interferogram shift
struct Config_t {
    size_t npts; ///< size of interferogram
    explicit Config_t( size_t n ): npts( n ) { }; ///< ctor
}; // struct Config_t
    
// these are useful enough to be promoted... no?
typedef  Connector::InputPort< Instrument::BlitzCplxDoubleVector > &JackIn;
typedef  Connector::OutputPort< Instrument::BlitzCplxDoubleVector > &JackOut;
typedef  Connector::InputPort< Config_t > &JackConfig;


/// Ports used by the IfgShift operator.

struct Ports {
    JackIn unshiftedInterferogram; ///< interferogram vector in
    JackOut shiftedInterferogram; ///< interferogram vector out
    JackConfig config; ///< configuration settings in (const)
    
    Ports( JackIn i, JackOut o, JackConfig c): 
        unshiftedInterferogram(i), shiftedInterferogram(o), config(c) 
    {}; ///< ctor
    ~Ports() {}; ///< dtor
};

/// \brief Concrete interferogram shift operator
/// 
/// transforms vector v_0 ... v_m-1 v_m ... v_npts 
/// into vector v_m ... v_npts v0 ... v_m-1
/// where m = npts/2
class IfgShiftOperator: Operator::AbstractOperator
{
private:
    Ports &ports;
public:
    IfgShiftOperator( Ports &p ): ports(p) { }; ///< ctor
    virtual ~IfgShiftOperator() { }; ///< dtor
    virtual Operator::Result operator() ( ); ///< make it so
}; // class Operator


} // namespace
#endif
