/**
 * \file SpectrumFold.cc
 * \brief SHIS implementation of SpectrumFold operator
 * \author Maciek Smuga-Otto <maciek@ssec.wisc.edu>
 *
 * based directly on the Scanning-HIS rsh2fbf application, written by Ray Garcia <rayg@ssec.wisc.edu>
 *
 *  \version $Id: SpectrumFold.cc,v 1.10 2004/10/07 15:53:33 maciek Exp $
 *
 *  \par Copyright:
 *  \verbatim
 *
 *  Copyright UW/SSEC, ALL RIGHTS RESERVED, 2004
 *  Space Science and Engineering Center
 *  University of Wisconsin - Madison, USA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 *  \endverbatim
 */

#include "SpectrumFold.h"

namespace SpecFold
{

Operator::Result SpecFoldOperator::operator() ( )
{
    const Instrument::BlitzCplxDoubleVector &in( ports.prefoldedSpectrum );
    Instrument::BlitzCplxDoubleVector &out( ports.foldedSpectrum );
    const Config_t &config( ports.config );
    
    size_t N = config.npts;
    int poldFoint = N - config.foldPoint - 1;
    
    if ( !N || N < config.foldPoint ) 
        return Operator::WARNING;
    int dest = 0;
    for (int src = poldFoint; src >= 0; src--, dest++) {
        out(dest) = in(src);
    }
    // dest is now at poldFoint
    for (int src = N-1; src > poldFoint; src--, dest++) {
        out(dest) = in(src);
    }
    return Operator::OK;
}; // operator() 
        
} // namespace

/******************** original function body in rsh2fbf:
int sol1SpcFold( solContent &out, const solContent &in, unsigned foldPoint ) { 
    // flip source end for end, around fold point
    // for poldFoint = N-1 - foldPoint...:
    // a...b <poldFoint> c....d transforms to
    // b...a <poldFoint> d....c
    
    out.resize( in.size() );
    int N = in.size() / 2;    
    int poldFoint=N-1-foldPoint;
    if (!N || N<(int)foldPoint) return 1;
    // unsigned src, dest;
    int src, dest = 0;
    for( src = poldFoint; src>=0; src--,dest++ ) {      
        out[dest*2] = in[src*2];
        out[(dest*2)+1] = in[(src*2)+1];
    }
    // dest = poldFoint...
    for( src=N-1; src>poldFoint; src--, dest++ ) {
        out[dest*2] = in[src*2]; 
        out[(dest*2)+1] = in[(src*2)+1];
    }
    return 0;
}

int sol1SpcFold( solComplexRads &out, const solComplexRads &in, sol1SpcFold_status &s ) {
    int rc = 0;
    for( unsigned i=0; i<IFG4_BANDS; i++ ) 
        if (!s.suppress[i]) 
            rc |= sol1SpcFold( out.qi[i], in.qi[i], s.foldPoint[i] );
    return rc;
}
*********************/

